//
// Copyright(C) 2007-2014 Samuel Villarreal
// Copyright(C) 2015 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#pragma once

//[BP]
#define FOG_RANGED

#ifdef FOG_RANGED
	#define FOG_DEPTH length(inVar(input, out_position))
#else //LINEAR
	#define FOG_DEPTH -inVar(input, out_position).z
#endif

#define ATTRIB_POSITION     0
#define ATTRIB_TEXCOORD     1
#define ATTRIB_COLOR        2
#define ATTRIB_NORMAL       3

#ifndef HLSL
    #define mul(m1, m2) (m1 * m2)
    #define static
    #define saturate(a) clamp(a, 0.0, 1.0)
    #define cast(type, n) type(n)
#else
    #define vec2 float2
    #define vec3 float3
    #define vec4 float4
    #define mat2 float2x2
    #define mat3 float3x3
    #define mat4 float4x4
    
    #define ivec2 int2
    #define ivec3 int3
    #define ivec4 int4
    
    #define cast(type, n) (type)n
    
    #define fract(x) ((x) - floor((x)))
    #define mix(x, y, a) ((x)*(1.0-(a))+(y)*(a))
    #define dFdx(x) ddx(x)
    #define dFdy(y) ddy(y)
    
    #define inversesqrt(x) rsqrt(x)
    
#endif

#ifndef EMULATE_UNIFORM_BUFFERS
    #define begin_cbuffer(name, register)   \
    uniform_buffer(name, register) \
    {
        
    #define cbuffer_member(type, name)  type name
    #define end_cbuffer() };
#else
    #define begin_cbuffer(name, register)
    #define cbuffer_member(type, name) uniform type name
    #define end_cbuffer()
#endif

#ifdef HLSL

    #define begin_input(name)                           struct name {
    #define end_input                                   };

    #define begin_output(name)                          struct name {
    #define end_output                                  };
    
    #define uniform_buffer(name, block)                 cbuffer name// : register(cb##block)
    
    #define hlslAttribName1(idx)                        attrib##idx
    #define hlslAttribName2(idx)                        ATTRIB##idx
    
    #define var_attrib(idx, type)                       type hlslAttribName1(idx) : hlslAttribName2(idx)
    #define inVarAttrib(idx, structName)                structName.hlslAttribName1(idx)

    #define def_var_out(type, name, gpuName)            type name : gpuName;
    #define def_var_in(type, name, gpuName)             type name : gpuName;
    #define def_var_outPosition(name)                   def_var_out(vec4, name, SV_POSITION)
    #define def_var_position(name)                      def_var_in(vec4, name, SV_POSITION)
    #define def_var_fragment(name)                      def_var_out(vec4, name, SV_TARGET)
    #define def_var_pixelTarget(type, idx)              type outPixelTarget##idx : SV_Target##idx;
    #define def_var_outClipPlane()                      def_var_out(float, g_clipplane0, SV_ClipDistance0)
    #define def_var_clipPlane()                         def_var_in(float, g_clipplane0, SV_ClipDistance0)

    #define shader_main(outStruct, inStruct, inName)    outStruct main(inStruct inName)
    #define shader_pixel(outStruct, inStruct, inName)   outStruct main(float4 inFragCoord : SV_Position, inStruct inName)
    
    #define declareOutVar(outStruct, name)              outStruct name;
    #define outVar(name, field)                         name.field
    #define outVarPosition(name, field)                 name.field
    #define outVarFragment(name, field)                 name.field
    #define outVarClipPlane(name, idx)                  name.g_clipplane##idx
    #define outVarPixelTarget(name, idx)                name.outPixelTarget##idx
    #define inVar(inName, field)                        inName.field
    #define outReturn(name)                             return name;
    
    #define def_sampler(dim, idx)   \
        Texture##dim uTex##idx : register(t##idx); \
        SamplerState uTexSample##idx : register(s##idx)
    
    #define sample_texture(idx, coord)                  uTex##idx.SampleLevel (uTexSample##idx, coord, 0)
    #define sample_textureNoLOD(idx, coord)             uTex##idx.Sample (uTexSample##idx, coord)
    #define load_texture(idx, coord, mip)               uTex##idx.Load (ivec3(coord, mip))
    
    #define M(mtx, col, row)                            mtx[row][col]
    #define MV4(mtx, col)                               vec4(mtx[0][col], mtx[1][col], mtx[2][col], mtx[3][col])
    #define MV3(mtx, col)                               vec3(mtx[0][col], mtx[1][col], mtx[2][col])
    #define MV2(mtx, col)                               vec2(mtx[0][col], mtx[1][col])

#else // GLSL

    #if GLSL_VERSION >= 330
        #define glslAttrib(idx, type) layout(location = idx) in type attrib##idx
    #elif GLSL_VERSION >= 130
        #define glslAttrib(idx, type) in type attrib##idx
    #else
        #define glslAttrib(idx, type) attribute type attrib##idx
    #endif
    
    #define glslAttribName(idx)                         attrib##idx

    #define begin_input(name)
    #define end_input

    #define begin_output(name)
    #define end_output
    
    #define uniform_buffer(name, block)                 layout (std140) uniform name
    
    #define var_attrib(idx, type)                       glslAttrib(idx, type)
    #define inVarAttrib(idx, structName)                glslAttribName(idx)

    #if GLSL_VERSION < 130
        #define glsl_out    varying
        #define glsl_in     varying
    #else
        #define glsl_out    out
        #define glsl_in     in
    #endif

    #define def_var_out(type, name, gpuName)            glsl_out type name;
    #define def_var_in(type, name, gpuName)             glsl_in type name;
    #define def_var_outPosition(name)
    #define def_var_position(name)
    #if GLSL_VERSION < 300
    #define def_var_fragment(name)
    #else
    #define def_var_fragment(name)
                     glsl_out vec4 name;
    #endif
    #define def_var_pixelTarget(type, idx)
    #define def_var_outClipPlane()
    #define def_var_clipPlane()

    #define shader_main(outStruct, inStruct, inName)    void main()
    #define shader_pixel(outStruct, inStruct, inName)   void main()
    
    #define declareOutVar(outStruct, name)
    #define outVar(name, field)                         field
    #define outVarPosition(name, field)                 gl_Position
    #if GLSL_VERSION < 300
    #define outVarFragment(name, field)                 gl_FragColor
    #define outVarPixelTarget(name, idx)                gl_FragData[idx]
    #else
    #define outVarFragment(name, field)                 field
    #define outVarPixelTarget(name, idx)                field[idx]
    #endif
    #define outVarClipPlane(name, idx)                  gl_ClipDistance[idx]
    #define inVar(inName, field)                        field
    #define outReturn(name)                             return;
    
    #define def_sampler(dim, idx)                       uniform sampler##dim uTex##idx
    #define sample_texture(idx, coord)                  texture(uTex##idx, coord)
    #define sample_textureNoLOD(idx, coord)             sample_texture(idx, coord)
    #define load_texture(idx, coord, mip)               texelFetch(uTex##idx, coord, mip)
    
    #define inFragCoord                                 gl_FragCoord
    
    #define M(mtx, col, row)                            mtx[col][row]
    #define MV4(mtx, col)                               mtx[col]
    #define MV3(mtx, col)                               mtx[col].xyz
    #define MV2(mtx, col)                               mtx[col].xy

#endif // HLSL
