//
// Copyright(C) 2007-2014 Samuel Villarreal
// Copyright(C) 2015 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#pragma once

#include "progs/T1/common.inc"
#include "progs/T1/globals_structs.inc"

IMPLEMENT_UNIFORM_BUFFER_RENDERVIEW();

//-----------------------------------------------------------------------------
//
// Built-in shader uniforms
//
//-----------------------------------------------------------------------------

uniform vec4        uClipPlane0;
uniform vec4        uClipPlane1;
uniform vec4        uClipPlane2;
uniform vec4        uClipPlane3;

//-----------------------------------------------------------------------------
//
// Common material uniforms
//
//-----------------------------------------------------------------------------

uniform int         uReflective;
uniform vec3        uLightDirection;
uniform vec4        uLightDirectionColor;

//-----------------------------------------------------------------------------
//
// Common functions
//
//-----------------------------------------------------------------------------

#ifdef KEX_XBOXONE
#define SETMODELVIEWMATRIX(outMatrix)    \
    M(outMatrix, 0, 0) = inVarAttrib(4, input).x;   \
    M(outMatrix, 0, 1) = inVarAttrib(4, input).y;   \
    M(outMatrix, 0, 2) = inVarAttrib(4, input).z;   \
    M(outMatrix, 0, 3) = inVarAttrib(4, input).w;   \
    M(outMatrix, 1, 0) = inVarAttrib(5, input).x;   \
    M(outMatrix, 1, 1) = inVarAttrib(5, input).y;   \
    M(outMatrix, 1, 2) = inVarAttrib(5, input).z;   \
    M(outMatrix, 1, 3) = inVarAttrib(5, input).w;   \
    M(outMatrix, 2, 0) = inVarAttrib(6, input).x;   \
    M(outMatrix, 2, 1) = inVarAttrib(6, input).y;   \
    M(outMatrix, 2, 2) = inVarAttrib(6, input).z;   \
    M(outMatrix, 2, 3) = inVarAttrib(6, input).w;   \
    M(outMatrix, 3, 0) = inVarAttrib(7, input).x;   \
    M(outMatrix, 3, 1) = inVarAttrib(7, input).y;   \
    M(outMatrix, 3, 2) = inVarAttrib(7, input).z;   \
    M(outMatrix, 3, 3) = inVarAttrib(7, input).w;   \
    outMatrix = mul(uModelViewMatrix, outMatrix)
#else
#define SETMODELVIEWMATRIX(outMatrix)  \
    outMatrix = uModelViewMatrix
#endif

// ----------------------------------------------------
vec2 SphereMap(const in vec3 vert, const in vec3 normal)
{
    vec3 r = reflect(vert, normal);
    r.z += 1.0;
    r = normalize(r);
    
    return r.xy * 0.5 + 0.5;
}
