//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX

#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(0, vec3);
    var_attrib(1, vec2);
    var_attrib(2, vec4);
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2,  out_texcoord, TEXCOORD0)
    def_var_out(vec4,  out_color,    COLOR0)
    def_var_out(vec3,  out_position, POSITION0)
    def_var_out(float, out_distance, SKYDIST0)
end_output

uniform int uTime;
uniform vec4 uSkyPlane;

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    vec4 vPosition                      = mul(uModelViewMatrix, vertex);
    outVarPosition(output, position)    = mul(uProjectionMatrix, vPosition);
    vec2 texcoord                       = inVarAttrib(1, input);
    outVar(output, out_color)           = inVarAttrib(2, input);
    outVar(output, out_position)        = vPosition.xyz;
    outVar(output, out_distance)        = -dot(vPosition, uSkyPlane);
    
    float time = float(uTime);
    texcoord.x += (time * 0.384) * -0.00005;
    texcoord.y += (time * 0.768) * -0.00005;
    
    
    outVar(output, out_texcoord)        = texcoord;
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/T1/globals_fragment.inc"

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2,  out_texcoord, TEXCOORD0)
    def_var_in(vec4,  out_color,    COLOR0)
    def_var_in(vec3,  out_position, POSITION0)
    def_var_in(float, out_distance, SKYDIST0)
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

//----------------------------------------------------
shader_pixel(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    float skyDepth = -(inVar(input, out_position).z / uZFar);
    float depth = LinearizeDepth(0.1, uFogFar, sample_texture(1, ScreenCoords()).r);

    if(skyDepth > depth)
    {
        discard;
        outReturn(output)
    }
    
    float alpha = inVar(input, out_color).a;
    vec4 tColor = sample_texture(0, inVar(input, out_texcoord)) * uDiffuseColor;
    
    alpha *= skyDepth;
    
    float threshold = clamp(smoothstep(0.0, 0.5, depth - skyDepth) * (1.0 * (1.0 - skyDepth)), 0.0, 1.0);
    outVarFragment(output, fragment) = vec4(tColor.rgb, clamp(pow(threshold, 0.5), 0.0, 1.0) * smoothstep(0.0, 0.3, alpha));
    
    outReturn(output)
}

#endif
