//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX
#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(ATTRIB_POSITION, vec3);
    var_attrib(ATTRIB_TEXCOORD, vec2);
    var_attrib(ATTRIB_NORMAL, vec3);
    var_attrib(ATTRIB_COLOR, vec4);
#if KEX_XBOXONE
    var_attrib(4, vec4);
    var_attrib(5, vec4);
    var_attrib(6, vec4);
    var_attrib(7, vec4);
    var_attrib(8, vec4);
#endif
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord, TEXCOORD0)
    def_var_out(vec4, out_color,    COLOR0)
    def_var_out(vec4, directionalColor, DIRCOLOR)
    def_var_out(vec3, out_position, POSITION0)
    def_var_outClipPlane()
end_output

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    mat4 outMatrix;
    SETMODELVIEWMATRIX(outMatrix);
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    vec4 vPosition                      = mul(outMatrix, vertex);
    outVarPosition(output, position)    = mul(uProjectionMatrix, vPosition);
    outVar(output, out_texcoord)        = inVarAttrib(ATTRIB_TEXCOORD, input);
    outVar(output, out_color)           = inVarAttrib(ATTRIB_COLOR, input);
    outVar(output, out_position)        = vPosition.xyz;
    
    vec3 vNormal                        = normalize(mul(cast(mat3, mul(outMatrix, uNormalMatrix)), inVarAttrib(3, input)));
    outVar(output, directionalColor)    = (uLightDirectionColor * max(dot(vNormal, uLightDirection), 0.0));
    
#if GLSL_VERSION >= 130 || HLSL
    outVarClipPlane(output, 0)          = dot(mul(outMatrix, vertex), uClipPlane0);
#endif
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/T1/globals_fragment.inc"

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord,      TEXCOORD0)
    def_var_in(vec4, out_color,         COLOR0)
    def_var_in(vec4, directionalColor,  DIRCOLOR)
    def_var_in(vec3, out_position, POSITION0)
    def_var_clipPlane()
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

float overlayBlend(const in float c, const in float base)
{
    if(base < 0.5)
    {
        return 2.0 * base * c;
    }
    
    return 1.0 - 2.0 * (1.0 - c) * (1.0 - base);
}

float hardlightBlend(const in float c, const in float base)
{
    if(c < 0.5)
    {
        return 2.0 * base * c;
    }
    
    return 1.0 - 2.0 * (1.0 - c) * (1.0 - base);
}

//----------------------------------------------------
shader_pixel(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    vec4 base = sample_texture(0, inVar(input, out_texcoord));
    vec4 color1;
    vec4 color2;
    
    color1.r = overlayBlend(uDiffuseColor.r, base.r);
    color1.g = overlayBlend(uDiffuseColor.g, base.g);
    color1.b = overlayBlend(uDiffuseColor.b, base.b);
    color1.a = overlayBlend(uDiffuseColor.a, base.a);
    
    color2.r = hardlightBlend(uDiffuseColor.r, color1.r);
    color2.g = hardlightBlend(uDiffuseColor.g, color1.g);
    color2.b = hardlightBlend(uDiffuseColor.b, color1.b);
    color2.a = hardlightBlend(uDiffuseColor.a, color1.a);
    
    vec4 frag = mix(color1, color2, 0.5);
    float z = FOG_DEPTH;
    
    if(uFullBright == 1)
    {
        outVarFragment(output, fragment) = vec4(ComputeFog(z, frag).xyz, base.a);
        outReturn(output)
    }
    
    vec4 color = frag * uLightAmbience;
    vec3 rgb = (color + (frag * inVar(input, directionalColor))).rgb;
    
    outVarFragment(output, fragment) = vec4(ComputeFog(z, vec4(rgb, 1.0)).rgb, base.a);
    outReturn(output)
}

#endif
