//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX

#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(0, vec3);
    var_attrib(1, vec2);
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord, TEXCOORD0)
end_output

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    outVarPosition(output, position)    = mul(uProjectionMatrix, mul(uModelViewMatrix, vertex));
    outVar(output, out_texcoord)        = inVarAttrib(1, input);
    
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL

#include "progs/T1/globals_fragment.inc"

uniform float uMaxSpan;
uniform float uReduceMax;
uniform float uReduceMin;

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord, TEXCOORD0)
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

//----------------------------------------------------
shader_main(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    vec2 texcoord = inVar(input, out_texcoord);
    vec2 frameBufSize = vec2(float(uViewWidth), float(uViewHeight));
    
    vec3 rgbNW = sample_texture(0, texcoord + (vec2(-1.0, -1.0) / frameBufSize)).xyz;
    vec3 rgbNE = sample_texture(0, texcoord + (vec2( 1.0, -1.0) / frameBufSize)).xyz;
    vec3 rgbSW = sample_texture(0, texcoord + (vec2(-1.0,  1.0) / frameBufSize)).xyz;
    vec3 rgbSE = sample_texture(0, texcoord + (vec2( 1.0,  1.0) / frameBufSize)).xyz;
    vec3 rgbM  = sample_texture(0, texcoord).xyz;
    
    vec3 luma = vec3(0.299, 0.587, 0.114);
    
    float lumaNW = dot(rgbNW, luma);
    float lumaNE = dot(rgbNE, luma);
    float lumaSW = dot(rgbSW, luma);
    float lumaSE = dot(rgbSE, luma);
    float lumaM  = dot(rgbM,  luma);
    
    float lumaMin = min(lumaM, min(min(lumaNW, lumaNE), min(lumaSW, lumaSE)));
    float lumaMax = max(lumaM, max(max(lumaNW, lumaNE), max(lumaSW, lumaSE)));
    
    vec2 dir;
    dir.x = -((lumaNW + lumaNE) - (lumaSW + lumaSE));
    dir.y =  ((lumaNW + lumaSW) - (lumaNE + lumaSE));
    
    float dirReduce = max((lumaNW + lumaNE + lumaSW + lumaSE) * (0.25 * (1.0 / uReduceMax)),
                          (1.0 / uReduceMin));
    
    float rcpDirMin = 1.0 / (min(abs(dir.x), abs(dir.y)) + dirReduce);
    
    dir = min(vec2(uMaxSpan, uMaxSpan),
              max(vec2(-uMaxSpan, -uMaxSpan),
                  dir * rcpDirMin)) / frameBufSize;
    
    vec3 rgbA = (1.0/2.0) * (sample_texture(0, texcoord + dir * (1.0/3.0 - 0.5)).xyz +
                             sample_texture(0, texcoord + dir * (2.0/3.0 - 0.5)).xyz);
    
    vec3 rgbB = rgbA * (1.0/2.0) + (1.0/4.0) * (sample_texture(0, texcoord + dir * (0.0/3.0 - 0.5)).xyz +
                                                sample_texture(0, texcoord + dir * (3.0/3.0 - 0.5)).xyz);
    float lumaB = dot(rgbB, luma);
    
    if((lumaB < lumaMin) || (lumaB > lumaMax))
    {
        outVarFragment(output, fragment) = vec4(rgbA, 1.0);
        outReturn(output)
    }
    
    outVarFragment(output, fragment) = vec4(rgbB, 1.0);
    outReturn(output)
}

#endif
