//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX
#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(ATTRIB_POSITION, vec3);
    var_attrib(ATTRIB_TEXCOORD, vec2);
    var_attrib(ATTRIB_NORMAL, vec3);
    var_attrib(ATTRIB_COLOR, vec4);
#if KEX_XBOXONE
    var_attrib(4, vec4);
    var_attrib(5, vec4);
    var_attrib(6, vec4);
    var_attrib(7, vec4);
    var_attrib(8, vec4);
#endif
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord, TEXCOORD0)
    def_var_out(vec4, out_color,    COLOR0)
    def_var_out(float, testDepth, TESTDEPTH0)
    def_var_out(vec3, out_position,     POSITION0)
    def_var_outClipPlane()
end_output

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    mat4 outMatrix;
    SETMODELVIEWMATRIX(outMatrix);
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    vec4 vPosition                      = mul(outMatrix, vertex);
    outVarPosition(output, position)    = mul(uProjectionMatrix, vPosition);
    outVar(output, out_texcoord)        = inVarAttrib(ATTRIB_TEXCOORD, input);
    outVar(output, out_color)           = inVarAttrib(ATTRIB_COLOR, input);
    outVar(output, out_position)        = vPosition.xyz;
    
    outVar(output, testDepth)           = outVarPosition(output, position).z;
    
#if GLSL_VERSION >= 130 || HLSL
    outVarClipPlane(output, 0)          = dot(mul(outMatrix, vertex), uClipPlane0);
#endif
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/T1/globals_fragment.inc"

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord, TEXCOORD0)
    def_var_in(vec4, out_color,    COLOR0)
    def_var_in(vec3, out_position, POSITION0)
    def_var_clipPlane()
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

uniform float uFarRange;
uniform float uNearRange;
uniform float uScale;
uniform float uPower;

float linearizeDepth(float near, float far, float depth)
{
    return 2.0 * near / (far + near - depth * (far - near));
}

float contrast(float d)
{
    float val = clamp(2.0 * ((d > 0.5) ? 1.0-d : d), 0.0, 1.0);

    float a = 0.5 * pow(val, 2.0);
    
    return (d > 0.5) ? 1.0 - a : a;
}

//----------------------------------------------------
shader_pixel(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    vec4 frag = sample_texture(0, inVar(input, out_texcoord));
    vec2 scrn = ScreenCoords();
    
    float zdepth = FOG_DEPTH;
    float depth = 1.0 - linearizeDepth(uNearRange, uFarRange, sample_texture(1, scrn).r);
    
    vec4 base = frag * uDiffuseColor;
    
    outVarFragment(output, fragment) = vec4(ComputeFog(zdepth, base).rgb, base.a);
    outReturn(output)
}

#endif
