//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX

#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(0, vec3);
    var_attrib(1, vec2);
    var_attrib(2, vec4);
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord, TEXCOORD0)
    def_var_out(vec4, out_color,    COLOR0)
end_output

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    outVarPosition(output, position)    = mul(uProjectionMatrix, mul(uModelViewMatrix, vertex));
    outVar(output, out_texcoord)        = inVarAttrib(1, input);
    outVar(output, out_color)           = inVarAttrib(2, input);
    
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/T1/globals_fragment.inc"

uniform float uExposure;
uniform vec3 uLightCoordinate;

static const int NUM_SAMPLES       = 100;

static const float SCATTER_DECAY   = 1.0;
static const float SCATTER_DENSITY = 0.84;
static const float SCATTER_WEIGHT  = 3.5;

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord, TEXCOORD0)
    def_var_in(vec4, out_color,    COLOR0)
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

//----------------------------------------------------
shader_main(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    vec2 dir = uLightCoordinate.xy;
    dir.y = -(dir.y - 2.0 * 0.5);
    
    // blah
#ifdef HLSL
    dir.y = 1.0 - dir.y;
#endif
    
    if(dir.x < 0.0 || dir.y < 0.0) { discard; outReturn(output) }
    if(dir.x > 1.0 || dir.y > 1.0) { discard; outReturn(output) }
    
    vec2 deltaTextCoord = vec2(inVar(input, out_texcoord) - dir);
    
    outVarFragment(output, fragment) = vec4(sample_texture(0, inVar(input, out_texcoord)).rgb, 1.0);
    
    vec2 textCoo = inVar(input, out_texcoord);
    deltaTextCoord *= 1.0 /  float(NUM_SAMPLES) * SCATTER_DENSITY;
    float illuminationDecay = 1.0;

    for(int i = 0; i < NUM_SAMPLES; i++)
    {
        textCoo -= deltaTextCoord;
    
        vec4 sa = sample_texture(0, clamp(textCoo, 0.0, 1.0));
        
        sa *= illuminationDecay * SCATTER_WEIGHT;

        outVarFragment(output, fragment) += sa;
        illuminationDecay *= SCATTER_DECAY;
    }
    
    vec2 crd = vec2(2.0 * (dir.x - 0.5), 2.0 * (dir.y - 0.5));
    float d = clamp((1.0 - length(crd)) + 0.75, 0.0, 1.0);
     
    outVarFragment(output, fragment) *= (uExposure * d);
    outVarFragment(output, fragment) *= inVar(input, out_color);
    outReturn(output)
}

#endif
