//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX
#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(ATTRIB_POSITION, vec3);
    var_attrib(ATTRIB_TEXCOORD, vec2);
    var_attrib(ATTRIB_COLOR, vec4);
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord, TEXCOORD0)
    def_var_out(vec2, out_refractionCoord, TEXCOORD1)
    def_var_out(vec4, out_color, COLOR0)
end_output

uniform float uRefractionTime;
uniform float uReflectionScale;
uniform int uRefractionStage;

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    vec4 vertex                         = vec4(inVarAttrib(ATTRIB_POSITION, input), 1.0);
    outVarPosition(output, position)    = mul(uProjectionMatrix, mul(uModelViewMatrix, vertex));
    outVar(output, out_texcoord)        = inVarAttrib(ATTRIB_TEXCOORD, input);
    outVar(output, out_color)           = inVarAttrib(ATTRIB_COLOR, input);
    
    if(uRefractionStage == 0)
    {
        mat4 mtx;
        float time = uRefractionTime;
        
        M(mtx, 0, 0) = 1.0;
        M(mtx, 0, 1) = 0.0;
        M(mtx, 0, 2) = 0.0;
        M(mtx, 0, 3) = 0.0;
        M(mtx, 1, 0) = 0.0;
        M(mtx, 1, 1) = 1.0;
        M(mtx, 1, 2) = 0.0;
        M(mtx, 1, 3) = 0.0;
        M(mtx, 2, 0) = 0.0;
        M(mtx, 2, 1) = 0.0;
        M(mtx, 2, 2) = 1.0;
        M(mtx, 2, 3) = 0.0;
        M(mtx, 3, 0) = 0.0;
        M(mtx, 3, 1) = 0.0;
        M(mtx, 3, 2) = 0.0;
        M(mtx, 3, 3) = 1.0;
        
        float t = uReflectionScale;
        
        M(mtx, 0, 0) *= t;
        M(mtx, 0, 1) *= t;
        M(mtx, 0, 2) *= t;
        M(mtx, 1, 0) *= t;
        M(mtx, 1, 1) *= t;
        M(mtx, 1, 2) *= t;
        M(mtx, 2, 0) *= t;
        M(mtx, 2, 1) *= t;
        M(mtx, 2, 2) *= t;
        
        outVar(output, out_refractionCoord) = inVarAttrib(ATTRIB_TEXCOORD, input);
        outVar(output, out_refractionCoord).x -= 0.5;
        outVar(output, out_refractionCoord).y -= 0.5;
        
        float x = outVar(output, out_refractionCoord).x;
        float y = outVar(output, out_refractionCoord).y;
        
        outVar(output, out_refractionCoord).x = M(mtx, 1, 0) * y + M(mtx, 0, 0) * x + M(mtx, 3, 0);
        outVar(output, out_refractionCoord).y = M(mtx, 1, 1) * y + M(mtx, 0, 1) * x + M(mtx, 3, 1);
        
        outVar(output, out_refractionCoord).x += 0.5;
        outVar(output, out_refractionCoord).y += 0.5;
        
        outVar(output, out_refractionCoord).x += cos(time * 0.0001);
        outVar(output, out_refractionCoord).y += sin(time * 0.0001);
    }
    else
    {
        outVar(output, out_refractionCoord) = vec2(0.0, 0.0);
    }
    
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/T1/globals_fragment.inc"

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord, TEXCOORD0)
    def_var_in(vec2, out_refractionCoord, TEXCOORD1)
    def_var_in(vec4, out_color, COLOR0)
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

uniform float uReflectionScale;
uniform int uRefractionStage;

//----------------------------------------------------
shader_pixel(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    if(uRefractionStage == 1)
    {
        vec4 refraction = 2.0 * sample_texture(1, ScreenCoords()) - 1.0;
        vec2 texCoords = clamp(inVar(input, out_texcoord) + (refraction.xy * uReflectionScale), 0.0, 1.0);
        vec4 color = sample_texture(0, texCoords) * inVar(input, out_color);

        outVarFragment(output, fragment) = vec4(color.rgb, inVar(input, out_color).a);
    }
    else
    {
        outVarFragment(output, fragment) = sample_texture(0, inVar(input, out_refractionCoord));
    }
     
    outReturn(output)
}

#endif
