//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX

#include "progs/T1/globals_vertex.inc"

uniform mat4 uCameraTransform;

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(0, vec3);
    var_attrib(1, vec2);
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord, TEXCOORD0)
end_output

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    vec4 vertex = vec4(inVarAttrib(0, input), 1.0);
    
    outVarPosition(output, position) = mul(uProjectionMatrix, mul(uModelViewMatrix, mul(uCameraTransform, vertex)));
    outVar(output, out_texcoord) = inVarAttrib(1, input);
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/T1/globals_fragment.inc"

static const vec2 centerCoord = vec2(0.5, 0.5);

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord, TEXCOORD0)
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

//----------------------------------------------------
shader_main(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    float delta = (1.0 - distance(centerCoord, inVar(input, out_texcoord)) * 2.0) * 1.5;
    delta = clamp(delta, 0.0, 1.0);
    
    vec3 color1 = vec3(1.0, 1.0, 1.0);
    vec3 color2 = vec3(1.0, 0.8, 0.35);
    
    float d = pow(delta, 2.0);
    
    outVarFragment(output, fragment) = vec4(mix(color2, color1, d), pow(delta, 3.0));
    outReturn(output)
}

#endif
