//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX
#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(ATTRIB_POSITION, vec3);
    var_attrib(ATTRIB_TEXCOORD, vec2);
    var_attrib(ATTRIB_NORMAL, vec3);
    var_attrib(ATTRIB_COLOR, vec4);
#if KEX_XBOXONE
    var_attrib(4, vec4);
    var_attrib(5, vec4);
    var_attrib(6, vec4);
    var_attrib(7, vec4);
    var_attrib(8, vec4);
#endif
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord,     TEXCOORD0)
    def_var_out(vec4, out_color,        COLOR0)
#if KEX_XBOXONE
    def_var_out(vec4, out_envColor,     COLOR1)
#endif
    def_var_out(vec4, directionalColor, DIRCOLOR)
    def_var_out(vec3, out_position,     POSITION0)
    def_var_outClipPlane()
end_output

static const vec3 luminanceVector = vec3(0.2125, 0.7154, 0.0721);

//----------------------------------------------------
vec3 RandomPos(vec3 vertexPos)
{
    // Radius relative to ssR
    float alpha = float(5 + 0.5) * (1.0 / 11.0);
    float angle = alpha * (7.0 * 6.28) + dot(vertexPos, luminanceVector);

    return normalize(vec3(cos(angle), sin(angle), cos(angle + 2.42) * sin(angle + 6.25)));
}

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    mat4 outMatrix;
    SETMODELVIEWMATRIX(outMatrix);
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    vec4 vPosition                      = mul(outMatrix, vertex);
    outVarPosition(output, position)    = mul(uProjectionMatrix, vPosition);
    outVar(output, out_texcoord)        = inVarAttrib(ATTRIB_TEXCOORD, input);
    outVar(output, out_position)        = vPosition.xyz;
    
    vec3 vNormal                        = normalize(mul(cast(mat3, mul(outMatrix, uNormalMatrix)), inVarAttrib(3, input)));
    outVar(output, directionalColor)    = (uLightDirectionColor * max(dot(vNormal, uLightDirection), 0.0));
    
#if KEX_XBOXONE
    outVar(output, out_envColor)        = inVarAttrib(8, input);
#endif
    
#if GLSL_VERSION >= 130 || HLSL
    outVarClipPlane(output, 0)          = dot(mul(outMatrix, vertex), uClipPlane0);
#endif

    float alpha                         = inVarAttrib(ATTRIB_COLOR, input).a;
    outVar(output, out_color)           = vec4((RandomPos(vertex.xyz) + normalize(vertex.xyz + vNormal)) * 0.5 + 0.5, alpha);
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL
#include "progs/T1/globals_fragment.inc"

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2,    out_texcoord,       TEXCOORD0)
    def_var_in(vec4,    out_color,          COLOR0)
#if KEX_XBOXONE
    def_var_in(vec4,    out_envColor,       COLOR1)
#endif
    def_var_in(vec4,    directionalColor,   DIRCOLOR)
    def_var_in(vec3,    out_position,       POSITION0)
    def_var_clipPlane()
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

//----------------------------------------------------
shader_pixel(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    vec4 color = sample_textureNoLOD(0, inVar(input, out_texcoord));
    if(color.a <= 0.9)
    {
        discard;
        outReturn(output)
    }
    
    vec4 frag = color * inVar(input, out_color) * uDiffuseColor;
    float alpha = clamp(color.a * uDiffuseColor.a, 0.0, 1.0);
    float depth = FOG_DEPTH;
    
    frag.xyz = clamp(frag.xyz + ENV_COLOR.xyz, 0.0, 1.0);
    
    if(uFullBright == 1)
    {
        outVarFragment(output, fragment) = vec4(ComputeFog(depth, frag).xyz, alpha);
        outReturn(output)
    }
    
    vec4 amb = ((frag * uLightAmbience) * 2.0);
    amb = clamp(amb, 0.0, 1.0);
    
    vec4 finalColor = amb + (frag * inVar(input, directionalColor));
    outVarFragment(output, fragment) = vec4(ComputeFog(depth, finalColor).xyz, alpha * ENV_COLOR.a);
    outReturn(output)
}

#endif
