//
// Copyright(C) 2016-2017 Samuel Villarreal
// Copyright(C) 2016 Night Dive Studios, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//

#ifdef SHADER_VERTEX
#include "progs/T1/globals_vertex.inc"

//----------------------------------------------------
// input
begin_input(inVertex)
    var_attrib(0, vec3);
    var_attrib(1, vec2);
    var_attrib(2, vec4);
end_input

//----------------------------------------------------
// output
begin_output(outVertex)
    def_var_outPosition(position)
    def_var_out(vec2, out_texcoord, TEXCOORD0)
    def_var_out(vec4, out_color,    COLOR0)
    def_var_out(vec3, out_position, POSITION0)
end_output

//----------------------------------------------------
shader_main(outVertex, inVertex, input)
{
    declareOutVar(outVertex, output)
    
    vec4 vertex                         = vec4(inVarAttrib(0, input), 1.0);
    vec4 vPosition                      = mul(uModelViewMatrix, vertex);
    outVarPosition(output, position)    = mul(uProjectionMatrix, vPosition);
    outVar(output, out_texcoord)        = inVarAttrib(1, input);
    outVar(output, out_color)           = inVarAttrib(2, input);
    outVar(output, out_position)        = vPosition.xyz;
    
    outReturn(output)
}

#endif

#ifdef SHADER_PIXEL

#include "progs/T1/globals_fragment.inc"

//----------------------------------------------------
// input
begin_input(outVertex)
    def_var_position(position)
    def_var_in(vec2, out_texcoord, TEXCOORD0)
    def_var_in(vec4, out_color,    COLOR0)
    def_var_in(vec3, out_position, POSITION0)
end_input

//----------------------------------------------------
// output
begin_output(outPixel)
    def_var_fragment(fragment)
end_output

uniform int uGreenBlood;

//----------------------------------------------------
shader_pixel(outPixel, outVertex, input)
{
    declareOutVar(outPixel, output)
    
    vec4 frag = sample_texture(0, inVar(input, out_texcoord));
    
    if(frag.a <= 0.01)
    {
        discard;
        outReturn(output)
    }
    
    if(uGreenBlood == 1)
    {
        float tmp = frag.r;
        frag.r = frag.g;
        frag.g = tmp;
    }
    
    vec4 base = frag * inVar(input, out_color);
    vec4 fxColor = mix(uEnvironmentColor, base, frag.r);
    float p = sqrt(fxColor.r * fxColor.r * uEnvironmentColor.r +
                   fxColor.g * fxColor.g * uEnvironmentColor.g +
                   fxColor.b * fxColor.b * uEnvironmentColor.b);
                   
    p = clamp(p, 0.0, 1.0);
    
    fxColor.r = p + (fxColor.r - p) * 1.5;
    fxColor.g = p + (fxColor.g - p) * 1.5;
    fxColor.b = p + (fxColor.b - p) * 1.5;
    
    fxColor = clamp(fxColor + (uEnvironmentColor * 0.5), 0.0, 1.0);
    
    outVarFragment(output, fragment) = vec4(ComputeFog(FOG_DEPTH, fxColor).rgb, base.a);
    outReturn(output)
}

#endif
